From 38e127e631c63a5ab44ec5ae74ae89f15f928c3a Mon Sep 17 00:00:00 2001
From: josch <j.schauer@email.de>
Date: Wed, 12 Feb 2014 09:08:18 +0100
Subject: [PATCH 1/2] support new command line argument type Vector given as
 comma-separated list

---
 apt-pkg/contrib/cmndline.cc     | 16 ++++++++++++++++
 apt-pkg/contrib/cmndline.h      |  6 +++++-
 test/libapt/commandline_test.cc |  8 ++++++++
 3 files changed, 29 insertions(+), 1 deletion(-)

diff --git a/apt-pkg/contrib/cmndline.cc b/apt-pkg/contrib/cmndline.cc
index 2086d91..10cb3ce 100644
--- a/apt-pkg/contrib/cmndline.cc
+++ b/apt-pkg/contrib/cmndline.cc
@@ -249,6 +249,22 @@ bool CommandLine::HandleOpt(int &I,int argc,const char *argv[],
 	 
 	 return true;
       }
+
+      // comma separated item list
+      if ((A->Flags & Vector) == Vector)
+      {
+          const char *next;
+          const char *curr = Argument;
+          while ((next = strchr(curr, ',')) != NULL) {
+              if (next-curr == 1)
+                  return _error->Error(_("Option %s: Vector must not contain empty elements."),argv[I]);
+              Conf->Set(string(A->ConfName)+"::", string(curr, next-curr));
+              curr = next + 1;
+          }
+          // process last token
+          Conf->Set(string(A->ConfName)+"::", string(curr));
+          return true;
+      }
       
       const char *I = strchrnul(A->ConfName, ' ');
       if (*I == ' ')
diff --git a/apt-pkg/contrib/cmndline.h b/apt-pkg/contrib/cmndline.h
index 1802766..4066cae 100644
--- a/apt-pkg/contrib/cmndline.h
+++ b/apt-pkg/contrib/cmndline.h
@@ -37,6 +37,9 @@
      ArbItem    - Means the item is an arbitrary configuration string of
                   the form item=value, where item is passed directly
                   to the configuration class.
+     Vector     - Means the item is a comma separated list of items which
+                  are stored as a Vector in the configuration class.
+                  Implies HasArg.
    The default, if the flags are 0 is to use Boolean
    
    ##################################################################### */
@@ -73,7 +76,8 @@ class CommandLine
       Boolean = (1 << 2),
       InvBoolean = (1 << 3),
       ConfigFile = (1 << 4) | HasArg,
-      ArbItem = (1 << 5) | HasArg
+      ArbItem = (1 << 5) | HasArg,
+      Vector = (1 << 6) | HasArg
    };
 
    const char **FileList;
diff --git a/test/libapt/commandline_test.cc b/test/libapt/commandline_test.cc
index de8a30b..4951409 100644
--- a/test/libapt/commandline_test.cc
+++ b/test/libapt/commandline_test.cc
@@ -7,6 +7,7 @@ int main()
    CommandLine::Args Args[] = {
       { 't', 0, "Test::Worked", 0 },
       { 'z', "zero", "Test::Zero", 0 },
+      { 'v', "vector", "Test::Vector", CommandLine::Vector },
       {0,0,0,0}
    };
    CommandLine CmdL(Args,_config);
@@ -28,5 +29,12 @@ int main()
    equals(true, _config->FindB("Test::Worked", false));
    equals(false, _config->FindB("Test::Zero", false));
 
+   char const * argv3[] = { "test", "--vector", "one,two" };
+   CmdL.Parse(3, argv3);
+   std::vector<string> vec = _config->FindVector("Test::Vector");
+   equals(vec[0], "one");
+   equals(vec[1], "two");
+   equals(vec.size(), 2);
+
    return 0;
 }
-- 
1.8.5.3

